/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file AlbersEqualAreaConic.cpp
\brief Implementation file for a CAlbersEqualAreaConic class.

Implementation file for a CAlbersEqualAreaConic class.
<P>---------------------------------------------------------------------------*/




#include "AlbersEqualAreaConic.h"
#include "Geodetic.h"
#include "StdAfx.h"

/**--------------------------------------------------------------------------<BR>
CAlbersEqualAreaConic::CAlbersEqualAreaConic <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CAlbersEqualAreaConic::CAlbersEqualAreaConic(void)   : CProjection(eAlbersEqualAreaConic)
{
	m_dStandardParallel1 = conTHIRDPI;

	m_dStandardParallel2 = conTHIRDPI;

	m_dOriginLat = 0;

	m_dOriginLong = 0;

	CalculateConstants();
}

/**--------------------------------------------------------------------------<BR>
CAlbersEqualAreaConic::~CAlbersEqualAreaConic <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CAlbersEqualAreaConic::~CAlbersEqualAreaConic(void)
{
}

/**--------------------------------------------------------------------------<BR>
CAlbersEqualAreaConic::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CAlbersEqualAreaConic::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	double theta = m_dn * (dLongX - m_dOriginLong);

	double dP = sqrt( m_dC - 2 * m_dn * sin(dLatY)) / m_dn ;
	
	dLongX = dP * sin( theta );

	dLatY = m_dP0 - dP * cos( theta );
}


/**--------------------------------------------------------------------------<BR>
CAlbersEqualAreaConic::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CAlbersEqualAreaConic::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLatDegrees();
	dx = rLatLong.GetLongDegrees();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CAlbersEqualAreaConic::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CAlbersEqualAreaConic::InverseProject(double& dLatY, double& dLongX) const
{
	double theta = atan2(dLongX , (m_dP0 - dLatY));

	double dP = sqrt(dLongX * dLongX + ( m_dP0 - dLatY) * ( m_dP0 - dLatY));

	dLatY = asin( (m_dC - dP * dP * m_dn * m_dn ) / (2 * m_dn) );

	dLongX = m_dOriginLong + theta / m_dn;

	dLatY *= conDegreesPerRadian;

	dLongX *= conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CAlbersEqualAreaConic::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CAlbersEqualAreaConic::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = dY;

	double dLong = dX;

	InverseProject(dLat, dLong);

	rLatLong.SetLatDegrees(dLat);

	rLatLong.SetLongDegrees(dLong);
}



/**--------------------------------------------------------------------------<BR>
CAlbersEqualAreaConic::SetStandardParallel2 <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CAlbersEqualAreaConic::SetStandardParallels(double dStandardParallel, double dStandardParalle2)
{
	m_dStandardParallel1 = dStandardParallel * conRadiansPerDegree;

	m_dStandardParallel2 = dStandardParalle2 * conRadiansPerDegree;

	CalculateConstants();
}


/**--------------------------------------------------------------------------<BR>
CAlbersEqualAreaConic::SetStandardParallel2 <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CAlbersEqualAreaConic::SetOrigin(double dLat, double dLong)
{
	m_dOriginLat = dLat * conRadiansPerDegree;

	m_dOriginLong = dLong * conRadiansPerDegree;

	CalculateConstants();
}

/**--------------------------------------------------------------------------<BR>
CAlbersEqualAreaConic::CalculateConstants <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CAlbersEqualAreaConic::CalculateConstants()
{
	m_sin_SP1 = sin( m_dStandardParallel1 );

	m_sin_SP2 = sin( m_dStandardParallel2 );

	m_cos_SP1 = cos( m_dStandardParallel1 );

	m_dn = (m_sin_SP1 + m_sin_SP2) / 2;

	m_dC = m_cos_SP1 * m_cos_SP1 + 2 * m_dn * m_sin_SP1;

	m_dP0 = sqrt( m_dC - 2 * m_dn * sin(m_dOriginLat)) / m_dn ;

}