/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file CBonneProjection.cpp
\brief Implementation file for a CBonneProjection class.

Implementation file for a CBonneProjection class.
<P>---------------------------------------------------------------------------*/


#include "BonneProjection.h"
#include "Geodetic.h"
#include "StdAfx.h"

/**--------------------------------------------------------------------------<BR>
CBonneProjection::CBonneProjection <BR>
\brief 
<P>---------------------------------------------------------------------------*/
CBonneProjection::CBonneProjection(void) : CProjection(eBonneProjection)
{
	SetOrigin(40.0, 0);
}


/**--------------------------------------------------------------------------<BR>
CBonneProjection::~CBonneProjection <BR>
\brief 
<P>---------------------------------------------------------------------------*/
CBonneProjection::~CBonneProjection(void)
{
}


/**--------------------------------------------------------------------------<BR>
CBonneProjection::Project <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CBonneProjection::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	double dP = m_cot_SP + m_dStandardParallel - dLatY;

	double dE = (dLongX - m_dCentralMeridian) * cos(dLatY) / dP;

	dLongX = dP * sin(dE);

	dLatY = m_cot_SP - dP * cos( dE );
}


/**--------------------------------------------------------------------------<BR>
CCassini::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CBonneProjection::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();

	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CBonneProjection::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CBonneProjection::InverseProject(double& dLatY, double& dLongX) const
{
	double cot_SPLessY = m_cot_SP - dLatY;

	double dP;

	if (dLatY >= m_cot_SP)
		dP = -sqrt(dLongX * dLongX + cot_SPLessY * cot_SPLessY);
	else
		dP = sqrt(dLongX * dLongX + cot_SPLessY * cot_SPLessY);

	double dLat = m_cot_SP + m_dStandardParallel - dP;
	// Added a fix on the inverse which happens around the extemes of longitude 
	bool bfix = false;
	if (dLat > conHALFPI)
	{
		bfix = true;
		dLat = m_cot_SP + m_dStandardParallel + dP;
	}

	double dA = atan2( dLongX , ( m_cot_SP - dLatY) );

	double dLong = m_dCentralMeridian + (dP * dA ) / cos(dLat) ;

	if (bfix)
		dLong *= -1;

	dLatY = dLat * conDegreesPerRadian;

	dLongX = dLong * conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CBonneProjection::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CBonneProjection::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = dY;

	double dLong = dX;

	InverseProject(dLat, dLong);

	rLatLong.SetLatDegrees(dLat);

	rLatLong.SetLongDegrees(dLong);
}

/**--------------------------------------------------------------------------<BR>
CBonneProjection::SetStandardParellel <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CBonneProjection::SetOrigin(double dStandardParallel, double dCentralMeridian)
{
	m_dStandardParallel = dStandardParallel * conRadiansPerDegree;

	m_dCentralMeridian = dCentralMeridian * conRadiansPerDegree;

	m_cot_SP = 1 / tan(m_dStandardParallel);
}