/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file CCassini.cpp
\brief Implementation file for a CCassini class.

Implementation file for a CCassini class.
<P>---------------------------------------------------------------------------*/


#include "Cassini.h"
#include "Geodetic.h"
#include "StdAfx.h"



/**--------------------------------------------------------------------------<BR>
CCassini::CCassini <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
CCassini::CCassini(void)  : CProjection(eCassini)
{
	m_dStandardLongitude = 0;
}


/**--------------------------------------------------------------------------<BR>
CCassini::CCassini <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CCassini::~CCassini(void)
{
}

/**--------------------------------------------------------------------------<BR>
CCassini::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CCassini::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	double dX = asin( cos(dLatY) * sin(dLongX - m_dStandardLongitude));

	double dY = atan2( tan(dLatY) , cos(dLongX - m_dStandardLongitude));

	dLatY = dY;

	dLongX = dX;
}


/**--------------------------------------------------------------------------<BR>
CCassini::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CCassini::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();
	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CCassini::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CCassini::InverseProject(double& dLatY, double& dLongX) const
{
	double dD = dLatY + m_dStandardLongitude;

	double dLat = asin( sin(dD) * cos( dLongX) );

	double dLong = m_dStandardLongitude + atan2( tan(dLongX) , cos(dD));

	dLatY = dLat * conDegreesPerRadian;

	dLongX = dLong * conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CCassini::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CCassini::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dD = dY + m_dStandardLongitude;

	double dLat = asin( sin(dD) * cos( dX) );

	double dLong = m_dStandardLongitude + atan2( tan(dX) , cos(dD));

	rLatLong.SetLat(dLat);

	rLatLong.SetLong(dLong);
}

/**--------------------------------------------------------------------------<BR>
CCassini::SetStandardLongitude <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CCassini::SetStandardLongitude(double dStandardLongitude)
{
	m_dStandardLongitude = dStandardLongitude * conRadiansPerDegree;
}