/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file ConicEquidistant.cpp
\brief Implementation file for a CConicEquidistant class.

Implementation file for a CConicEquidistant class.
<P>---------------------------------------------------------------------------*/




#include "ConicEquidistant.h"
#include "Geodetic.h"
#include "StdAfx.h"



CConicEquidistant::CConicEquidistant(void)  : CProjection(eConicEquidistant)
{
	m_dStandardParallel1 = 0;

	m_dStandardParallel2 = conTHIRDPI;

	m_dOriginLat = 0;

	m_dOriginLong = 0;

	CalculateConstants();
}

CConicEquidistant::~CConicEquidistant(void)
{

}


/**--------------------------------------------------------------------------<BR>
CConicEquidistant::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CConicEquidistant::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	double dTheta = m_dn * (dLongX - m_dOriginLong);

	double dP = m_dG - dLatY;

	dLongX = dP * sin(dTheta);

	dLatY = m_dP0 - dP * cos(dTheta);
}


/**--------------------------------------------------------------------------<BR>
CConicEquidistant::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CConicEquidistant::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();
	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CConicEquidistant::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CConicEquidistant::InverseProject(double& dLatY, double& dLongX) const
{

	double dP = sqrt(dLongX * dLongX + (m_dP0 - dLatY) * (m_dP0 - dLatY) );
	if (m_dn < 0)
		dP = -dP;

//	double dTheta = atan(dLongX / (m_dP0 - dLatY) );
	double dTheta = atan2(dLongX , (m_dP0 - dLatY) );


	dLatY = m_dG - dP;

	dLongX = m_dOriginLong + dTheta / m_dn;

	dLatY *= conDegreesPerRadian;

	dLongX *= conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CConicEquidistant::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CConicEquidistant::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLatY = dY;

	double dLongX = dX;

	InverseProject(dLatY, dLongX);

	rLatLong.SetLat(dLatY);

	rLatLong.SetLong(dLongX);
}

/**--------------------------------------------------------------------------<BR>
CConicEquidistant::SetStandardParallel2 <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CConicEquidistant::SetStandardParallels(double dStandardParallel, double dStandardParalle2)
{
	m_dStandardParallel1 = dStandardParallel * conRadiansPerDegree;

	m_dStandardParallel2 = dStandardParalle2 * conRadiansPerDegree;

	CalculateConstants();
}


/**--------------------------------------------------------------------------<BR>
CConicEquidistant::SetStandardParallel2 <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CConicEquidistant::SetOrigin(double dLat, double dLong)
{
	m_dOriginLat = dLat * conRadiansPerDegree;

	m_dOriginLong = dLong * conRadiansPerDegree;

	CalculateConstants();
}

/**--------------------------------------------------------------------------<BR>
CConicEquidistant::CalculateConstants <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CConicEquidistant::CalculateConstants()
{
	double cos_SP1 = cos( m_dStandardParallel1 );

	m_dn = (cos_SP1 - cos( m_dStandardParallel2 ))/
				( m_dStandardParallel2 - m_dStandardParallel1);

	m_dG = cos_SP1 / m_dn +  m_dStandardParallel1;

	m_dP0 = m_dG - m_dOriginLat;
}