/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file EckertIV.cpp
\brief Implementation file for a CEckertIV class.

Implementation file for a CEckertIV class.
<P>---------------------------------------------------------------------------*/



#include "EckertIV.h"
#include "Geodetic.h"
#include "StdAfx.h"



CEckertIV::CEckertIV(void) : CProjection(eEckertIV)
{
	m_dStandardLongitude = 0;
}

CEckertIV::~CEckertIV(void)
{
}



/**--------------------------------------------------------------------------<BR>
CEckertIV::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CEckertIV::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	static const double dTolerance = 0.00000001;

	double theta1 = dLatY / 2.0; // Iteration start at lat / 2

	double theta0 = theta1 + dTolerance * 10.0;

	static const unsigned int nMaxIt = 50;

	unsigned int nIt = 0; 

	double sin_lat = sin (dLatY);

	while ( fabs(theta1 - theta0) > dTolerance)
	{
		theta0 = theta1;

		double dSinTheta = sin(theta0);

		double dCosTheta = cos(theta0);

		theta1 -= (  theta0 + dSinTheta * dCosTheta + 2.0*dSinTheta - (2.0 + conHALFPI) * sin_lat ) /
						( 2.0 * dCosTheta * ( 1.0 + dCosTheta)  );

		nIt++;
		if(nIt == nMaxIt)
		{
		//	assert(0);
			return;
		}
	}


	dLongX = 2.0 * (dLongX - m_dStandardLongitude) * (1.0 + cos( theta1 )) / 
					sqrt( conPI * (4.0 + conPI));

	dLatY = 2.0 * sqrt( conPI / (4.0 + conPI)) * sin( theta1 );
}



/**--------------------------------------------------------------------------<BR>
CEckertIV::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CEckertIV::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();

	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CEckertIV::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CEckertIV::InverseProject(double& dLatY, double& dLongX) const
{
	double sin_theta = dLatY * sqrt( (4.0 + conPI) / conPI) / 2.0;

	double theta = asin( sin_theta );

	double cos_theta = cos(theta);

	dLatY = asin(   (theta + sin_theta * cos_theta + 2.0 * sin_theta) /
						  (2.0 + conHALFPI)   );

//	m_dLong = dStandardLongitude + conPI * sqrt(4 + conPI) * Point.x / 
	//									(1 + dCosTheta);		// out by approx  * 2+ PI/2
	
	dLongX = m_dStandardLongitude + 1.00730304626 * (conPI * dLongX) / 
										((1.0 + cos_theta) * sqrt(1.0 + conHALFPI / 2.0)); // corrected.


	dLatY *= conDegreesPerRadian;

	dLongX *= conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CEckertIV::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CEckertIV::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLatY = dY;

	double dLongX = dX;

	InverseProject(dLatY, dLongX);

	rLatLong.SetLat(dLatY);

	rLatLong.SetLong(dLongX);
}


/**--------------------------------------------------------------------------<BR>
CEckertIV::SetStandardLongitude <BR>
\brief
<P>---------------------------------------------------------------------------*/
void CEckertIV::SetStandardLongitude(double dStandardLongitude)
{
	m_dStandardLongitude = dStandardLongitude * conRadiansPerDegree;

}