/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file EckertVI.cpp
\brief Implementation file for a CEckertVI class.

Implementation file for a CEckertVI class.
<P>---------------------------------------------------------------------------*/



#include "EckertVI.h"
#include "Geodetic.h"
#include "StdAfx.h"



CEckertVI::CEckertVI(void) : CProjection(eEckertVI)
{
	m_dStandardLongitude = 0;
}

CEckertVI::~CEckertVI(void)
{
}



/**--------------------------------------------------------------------------<BR>
CEckertVI::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CEckertVI::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;


	static const double dTolerance = 0.00000001;

	double theta1 = dLatY; // Iteration start at lat

	double theta0 = theta1 + dTolerance * 10;

	static const unsigned int nMaxIt = 50;

	unsigned int nIt = 0; 

	double sin_lat = sin (dLatY);

	while ( fabs(theta1 - theta0) > dTolerance)
	{
		theta0 = theta1;

		theta1 -= (  theta0 + sin(theta0) - (1 + conHALFPI) * sin_lat ) /
						( 1 + cos (theta0));

		nIt++;
		if(nIt == nMaxIt)
		{
		//	assert(0);

			return;
		}
	}

	double dDen = sqrt(2 + conPI);

	dLongX = (dLongX - m_dStandardLongitude) * (1 + cos (theta1)) /
					dDen;

	dLatY = 2 * theta1 / dDen;
}



/**--------------------------------------------------------------------------<BR>
CEckertVI::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CEckertVI::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();

	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CEckertVI::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CEckertVI::InverseProject(double& dLatY, double& dLongX) const
{
	double dA = sqrt( 2 + conPI);

	double theta = 0.5 * dA * dLatY;

	dLatY = asin(  (theta + sin(theta)) / ( 1 + conHALFPI));

	dLongX = m_dStandardLongitude + (dA * dLongX ) / (1 + cos(theta));


	dLatY *= conDegreesPerRadian;

	dLongX *= conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CEckertVI::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CEckertVI::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLatY = dY;

	double dLongX = dX;

	InverseProject(dLatY, dLongX);

	rLatLong.SetLat(dLatY);

	rLatLong.SetLong(dLongX);
}


/**--------------------------------------------------------------------------<BR>
CEckertVI::SetStandardLongitude <BR>
\brief
<P>---------------------------------------------------------------------------*/
void CEckertVI::SetStandardLongitude(double dStandardLongitude)
{
	m_dStandardLongitude = dStandardLongitude * conRadiansPerDegree;

}