/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file Geodetic.cpp
\brief Implementation file for the CGeoLatLong class which contains data for latitude, 
longitude and height. 
<P>---------------------------------------------------------------------------*/


#include "StdAfx.h"

#include "Geodetic.h"
#include "C3DPoint.h"
#include <stdio.h>
#include <string.h>



/**--------------------------------------------------------------------------<BR>
CGeoLatLong::CGeoLatLong <BR>
\brief Constructor initialises members to 0.
<P>---------------------------------------------------------------------------*/
CGeoLatLong::CGeoLatLong()
{
	m_dLat = 0;
	m_dLong = 0;
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::CGeoLatLong <BR>
\brief Constructor with assignment.
<P>---------------------------------------------------------------------------*/
CGeoLatLong::CGeoLatLong(double dLatitudeRadians, double dLongitudeRadians)
{
	m_dLat = dLatitudeRadians;
	m_dLong = dLongitudeRadians;
}

/**--------------------------------------------------------------------------<BR>
CGeoLatLong::CGeoLatLong <BR>
\brief Copy constructor.
<P>---------------------------------------------------------------------------*/
CGeoLatLong::CGeoLatLong(const CGeoLatLong& Other)
{
	// simple assignment
	m_dLat = Other.GetLat();
	m_dLong = Other.GetLong();
}

/**--------------------------------------------------------------------------<BR>
CGeoLatLong::~CGeoLatLong <BR>
\brief Destrcutor.
<P>---------------------------------------------------------------------------*/
CGeoLatLong::~CGeoLatLong()
{
}

/**--------------------------------------------------------------------------<BR>
CGeoLatLong::IsValid <BR>
\brief Checks for valid values.
<P>---------------------------------------------------------------------------*/
bool CGeoLatLong::IsValid(void) const
{
	return (m_dLat >= -conHALFPI && m_dLat <= conHALFPI &&
		m_dLong >= -conPI && m_dLong < conPI);
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::GetLatString <BR>
\brief Gets the latitude as a string. Target must be at least 10 chars long.
<P>---------------------------------------------------------------------------*/
void CGeoLatLong::GetLatString(char* dest) const
{
	sprintf_s( dest, 10, GetLatString(m_dLat * conDegreesPerRadian));
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::GetLongString <BR>
\brief Longitude as a string. Target must be at least 11 chars long.
<P>---------------------------------------------------------------------------*/
void CGeoLatLong::GetLongString(char* dest) const
{
	sprintf_s( dest, 11, GetLongString(m_dLong * conDegreesPerRadian));
}


/// Latitude as a string.
const char* CGeoLatLong::GetLatString() const
{
	return GetLatString(m_dLat * conDegreesPerRadian);
}

///// Longitude as a string. 
const char* CGeoLatLong::GetLongString() const
{
	return GetLongString(m_dLong * conDegreesPerRadian);
}

/// Latitude as a string. Target must be at least 10 chars long.
const char* CGeoLatLong::GetLatString(double dLatitudeDegrees)
{
	static char dest[20];

	int nSec = (int) (fabs( dLatitudeDegrees * 3600.0) + 0.5);

	if (nSec >= 0 && nSec <= (90 * 3600))
	{
		div_t dDeg = div(nSec, 3600);

		div_t dMin = div( dDeg.rem, 60);

		if(dLatitudeDegrees >= 0.0)
			sprintf_s( dest, 10, "N%02d:%02d:%02d",dDeg.quot, dMin.quot , dMin.rem );
		else
			sprintf_s( dest, 10,"S%02d:%02d:%02d", dDeg.quot, dMin.quot , dMin.rem);
	}
	else
	{
		sprintf_s( dest, 1, "" );
	}

	return dest;
}


///// Longitude as a string. Target must be at least 11 chars long.
const char* CGeoLatLong::GetLongString(double dLongitudeDegrees)
{
	static char dest[20];

	int nSec = (int) (fabs( dLongitudeDegrees * 3600.0) + 0.5);

	if (nSec >= 0 && nSec <= (180 * 3600))
	{
		div_t dDeg = div(nSec, 3600);

		div_t dMin = div( dDeg.rem, 60);

		if(dLongitudeDegrees >= 0.0)
			sprintf_s( dest, 11, "E%03d:%02d:%02d",dDeg.quot, dMin.quot , dMin.rem );
		else
			sprintf_s( dest, 11,"W%03d:%02d:%02d", dDeg.quot, dMin.quot , dMin.rem);
	}
	else
	{
		sprintf_s( dest, 1, "" );
	}

	return dest;
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::CGeoLatLong <BR>
\brief Assigment.
<P>---------------------------------------------------------------------------*/
CGeoLatLong& CGeoLatLong::operator=(const CGeoLatLong& Other)
{
	// Simple assignment
	m_dLat = Other.GetLat();
	m_dLong = Other.GetLong();
	// Return reference to this
	return *this;
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::operator== <BR>
\brief Equality test which is really a proximity.
<P>---------------------------------------------------------------------------*/
bool CGeoLatLong::operator==(CGeoLatLong& Other) const
{
	bool bLatClose;
	bool bLongClose;

	if ( m_dLat == 0 )
		bLatClose = Other.GetLat() == 0;
	else
		bLatClose = fabs(( Other.GetLat() - m_dLat ) / m_dLat ) 
												< conEqualityTolerance;
	if (!bLatClose)
		return false;		// Get out early if we can.



	if ( m_dLong == 0 )
		bLongClose = Other.GetLong() == 0;
	else
		bLongClose = fabs(( Other.GetLong() - m_dLong ) / m_dLong ) 
												< conEqualityTolerance;
	return (bLongClose);

}

/**--------------------------------------------------------------------------<BR>
CGeoLatLong::operator!= <BR>
\brief Inequality test which is really a proximity.
<P>---------------------------------------------------------------------------*/
bool CGeoLatLong::operator!=(CGeoLatLong& Other) const
{
	return (  !this->operator ==(Other) );	
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::Range <BR>
\brief Finds the range from one position to another based on sperical earth and
great circles. Range is in metres. Error between this and WSG84 slant range is
less than 1% at ranges up to 2000km. (Spherical Law of cosines).
<P>---------------------------------------------------------------------------*/
double CGeoLatLong::Range(const CGeoLatLong& Other) const
{
	double latO = Other.GetLat();
	double longO = Other.GetLong();

	if (m_dLat == latO && m_dLong == longO) return 0;

	double A = sin(m_dLat)*sin(latO) + 
		cos(m_dLat)*cos(latO)*cos(m_dLong - longO);

	return acos(A) * conEARTH_RADIUS_METRES;

}

/**--------------------------------------------------------------------------<BR>
CGeoLatLong::Heading <BR>
\brief Calculates the heading to another location in degrees.
<P>---------------------------------------------------------------------------*/
double CGeoLatLong::Heading(const CGeoLatLong& Other) const
{

	double latO = Other.GetLat();
	double longO = Other.GetLong();

	//check to see if they are on the same line of longitude
	if(m_dLong == longO)
	{
		// first check to see if the locations are the same
		if (m_dLat == latO) 
			return 0; //may as well return 0 as any other
	
		if(m_dLat > latO)
			return 180;
		else
			return 0;
	}

	if((fabs(m_dLat) == -conHALFPI) | (fabs(latO) == conHALFPI))
	{
		//check to see if the first is at the south pole or the second is at the north pole
		if((m_dLat == -conHALFPI) | (latO == conHALFPI))
			return 0;

		//check to see if the first is at the north pole or the second is at the south
		if((m_dLat == conHALFPI) | (latO ==-conHALFPI)) 
			return 180;
	}

	double sin_lat = sin(m_dLat);
	double sin_latO = sin(latO);
	double cos_lat = cos(m_dLat);
	double cos_latO = cos(latO);
	double cos_alpha = sin_latO * sin_lat + cos_latO * cos_lat * cos(longO - m_dLong);
	double alpha = acos(cos_alpha);

	double hdng;

	if(longO > m_dLong)
	{
		hdng = acos(  ( sin_latO - sin_lat * cos_alpha )/( cos_lat * sin(alpha) )  );
		if ( (longO - m_dLong) > conPI) 
		{
			// gone from e.g. e179 to w179
			hdng = conTWOPI - hdng;
		}
	}
	else
	{
		hdng = conTWOPI - acos(  ( sin_latO - sin_lat * cos_alpha )/( cos_lat * sin(alpha) )  );
		if ( (m_dLong - longO) > conPI) 
		{
			// gone from e.g. w179 to e179
			hdng = conTWOPI - hdng;
		}
	}

	if (hdng == conTWOPI) 
		return 0;

	return hdng;
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::RangeAndHeading <BR>
\brief Calculates the Range and heading to another location in degrees.
<P>---------------------------------------------------------------------------*/
void CGeoLatLong::RangeAndHeading(const CGeoLatLong& Other, 
					double& dRangeMetres, double& hdng) const
{
	double latO = Other.GetLat();
	double longO = Other.GetLong();

	if (m_dLat == latO && m_dLong == longO) 
	{
		dRangeMetres = 0;
		hdng = 0;
		return;
	}

	double sin_lat = sin(m_dLat);
	double sin_latO = sin(latO);
	double cos_lat = cos(m_dLat);
	double cos_latO = cos(latO);
	double cos_alpha = sin_latO * sin_lat + cos_latO * cos_lat * cos(longO - m_dLong);
	double alpha = acos(cos_alpha);

	dRangeMetres = alpha * conEARTH_RADIUS_METRES;


	//check to see if they are on the same line of longitude
	if(m_dLong == longO)
	{
		// No need to check if the locations are the same as done already.
		if(m_dLat > latO)
		{
			hdng = conPI;
			return;
		}
		else
		{
			hdng = 0;//may as well return 0 as any other
			return ; 
		}
	}

	if((fabs(m_dLat) == -conHALFPI) | (fabs(latO) == conHALFPI))
	{
		//check to see if the first is at the south pole or the second is at the north pole
		if((m_dLat == -conHALFPI) | (latO == conHALFPI))
		{
			hdng = 0;
			return;
		}

		//check to see if the first is at the north pole or the second is at the south
		if((m_dLat == conHALFPI) | (latO ==-conHALFPI)) 
		{
			hdng = conPI;
			return;
		}
	}

	if(longO > m_dLong)
	{
		hdng = acos(  ( sin_latO - sin_lat * cos_alpha )/( cos_lat * sin(alpha) )  );
		if ( (longO - m_dLong) > conPI) 
		{
			// gone from e.g. e179 to w179
			hdng = conTWOPI - hdng;
		}
	}
	else
	{
		hdng = conTWOPI - acos(  ( sin_latO - sin_lat * cos_alpha )/( cos_lat * sin(alpha) )  );
		if ( (m_dLong - longO) > conPI) 
		{
			// gone from e.g. w179 to e179
			hdng = conTWOPI - hdng;
		}
	}

	if (hdng == conTWOPI)
					hdng = 0;
}



/**--------------------------------------------------------------------------<BR>
CGeoLatLong::Set <BR>
\brief Set using a heading and a range from an origin. Range in metres. 
<P>---------------------------------------------------------------------------*/
void CGeoLatLong::Set(double dHeading, double dRange, const CGeoLatLong& Origin)
{
	// alpha is the angular Distance travelled round the earths surface
	double alpha = dRange /conEARTH_RADIUS_METRES;
	double sin_alpha = sin(alpha);
	double cos_alpha = cos(alpha);

	double latO = Origin.GetLat();
	double longO = Origin.GetLong();

	double sin_Olat = sin(latO);
	double cos_Olat = cos(latO);

	m_dLat = asin(  sin_Olat * cos_alpha + cos_Olat * sin_alpha * cos(dHeading)  );

	m_dLong = longO + atan2(sin(dHeading) * sin_alpha * cos_Olat , cos_alpha - sin_Olat * sin(m_dLat));

	while (m_dLong > conPI) 
		m_dLong -= conTWOPI;
	while (m_dLong < -conPI) 
		m_dLong += conTWOPI;

}





/**--------------------------------------------------------------------------<BR>
CGeoLatLong::Rotate <BR>
\brief Rotation.
<P>---------------------------------------------------------------------------*/
void CGeoLatLong::Rotate(const CGeoLatLong& Other)
{
	m_dLat += Other.GetLat();
	m_dLong += Other.GetLong();
	
	if (m_dLat < -conHALFPI)
		m_dLat = conPI + m_dLat;
		
	if (m_dLat > conHALFPI)
		m_dLat = -conPI + m_dLat;

	if (m_dLong < -conPI)
		m_dLong = conTWOPI + m_dLong;

	if (m_dLong > conPI)
		m_dLong = -conTWOPI + m_dLong;

}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::ConvertTo3DWSG84 <BR>
\brief Conversion to geocentric.
<P>---------------------------------------------------------------------------*/
void CGeoLatLong::GeocentricWSG84(C3DPoint& xyzPoint) const
{
	if(!IsValid())
		return;

	double Sin_Lat = sin(m_dLat);  //  sin(Latitude)
	double Cos_Lat = cos(m_dLat);  //  cos(Latitude)
	double Sin2_Lat = Sin_Lat * Sin_Lat; // Square of sin(Latitude)
	double Rn = conGeocent_Major / (sqrt(1.0e0 - conGeocent_e2 * Sin2_Lat)); //  Earth radius at location

	xyzPoint.x = Rn * Cos_Lat * cos(m_dLong);

	xyzPoint.y = Rn * Cos_Lat * sin(m_dLong);

	xyzPoint.z = (Rn * (1 - conGeocent_e2)) * Sin_Lat;

	return;
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::ConvertTo3D <BR>
\brief Conversion to geocentric.
<P>---------------------------------------------------------------------------*/
void CGeoLatLong::Geocentric(C3DPoint& xyzPoint) const
{
	if(!IsValid())
		return;

	double Sin_Lat = sin(m_dLat);  //  sin(Latitude)
	double Cos_Lat = cos(m_dLat);  //  cos(Latitude)

	xyzPoint.x = (conEARTH_RADIUS_METRES ) * Cos_Lat * cos(m_dLong);

	xyzPoint.y = (conEARTH_RADIUS_METRES ) * Cos_Lat * sin(m_dLong);

	xyzPoint.z = (conEARTH_RADIUS_METRES ) * Sin_Lat;

	return;
}

/**--------------------------------------------------------------------------<BR>
CGeoLatLong::SlantRangeWSG84 <BR>
\brief Slant Range.
<P>---------------------------------------------------------------------------*/
double CGeoLatLong::SlantRangeWSG84(const CGeoLatLong& Other) const
{
	C3DPoint ptThis;

	this->GeocentricWSG84(ptThis);

	C3DPoint ptOther;

	Other.GeocentricWSG84(ptOther);

	return ptThis.Distance(ptOther);

}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::SlantRange <BR>
\brief Slant Range.
<P>---------------------------------------------------------------------------*/
double CGeoLatLong::SlantRange(const CGeoLatLong& Other) const
{
	C3DPoint ptThis;

	this->Geocentric(ptThis);

	C3DPoint ptOther;

	Other.Geocentric(ptOther);

	return ptThis.Distance(ptOther);
}

/**--------------------------------------------------------------------------<BR>
CGeoLatLong::RangeToSlantRange <BR>
\brief Converts Range to Slant Range.
<P>---------------------------------------------------------------------------*/
double CGeoLatLong::RangeToSlantRange(double dRange)
{
	return 2* conEARTH_RADIUS_METRES * sin( dRange / (2 * conEARTH_RADIUS_METRES));

}
/**--------------------------------------------------------------------------<BR>
CGeoLatLong::SlantRangeToRange <BR>
\brief Converts Slant Range to Range.
<P>---------------------------------------------------------------------------*/
double CGeoLatLong::SlantRangeToRange(double dSlantRange)
{
	return 2* conEARTH_RADIUS_METRES *asin( dSlantRange / (2 * conEARTH_RADIUS_METRES));
}


double CGeoLatLong::LatitudeToDegrees(const char* pText)
{
	double dDegrees = 0.0;

	int nDegrees = 0;
    int nMinutes = 0;
    int nSeconds = 0;
	char chNeg;

	int nValues = sscanf_s( pText, "%c%02d:%02d:%02d",
                          &chNeg, sizeof(char), &nDegrees, &nMinutes, &nSeconds);

	if( nValues > 0 && nDegrees <= 90 && nMinutes < 60 && nSeconds < 60 )
    {
		dDegrees = nDegrees * 3600 + nMinutes * 60 + nSeconds;
    }

	dDegrees /= 3600;

	if(chNeg == 'S' || chNeg == 's')
		dDegrees = -dDegrees;

	return dDegrees;
}



double CGeoLatLong::LongitudeToDegrees(const char* pText)
{
	double dDegrees = 0.0;

	int nDegrees = 0;
    int nMinutes = 0;
    int nSeconds = 0;
	char chNeg;

	int nValues = sscanf_s( pText, "%c%03d:%02d:%02d",
                          &chNeg, sizeof(char), &nDegrees, &nMinutes, &nSeconds);

	if( nValues > 0 && nDegrees <= 180 && nMinutes < 60 && nSeconds < 60 )
    {
		dDegrees = nDegrees * 3600 + nMinutes * 60 + nSeconds;
    }

	dDegrees /= 3600;

	if(chNeg == 'W' || chNeg == 'w')
		dDegrees = -dDegrees;

	return dDegrees;
}











bool CGeoLatLong::ValidateLatitude(const char* strText)
{
    const int nStrLen = strlen(strText);

	if( nStrLen == 0 )
		return false;

    if( nStrLen < 9 || nStrLen > 11 )
        return false;

	if(strText[0] != 'N' && strText[0] != 'S')
		return false;

	// First numeric must be less than or equal to 9
	if(strText[1] < '0' || strText[1] > '9')
		return false;

	// Second numeric must be less than or equal to 9
	if(strText[2] < '0' || strText[2] > '9')
		return false;

	// If first numeric is 9 then don't allow second digit to be > 0
	if(((int)strText[1] - 48 == 9) && (strText[2] > '0'))
		return false;


	// Third char must be a colon
	if(strText[3] != ':')
		return false;

	// Fourth numeric must be less than or equal to 5
	if(strText[4] < '0' || strText[4] > '5')
		return false;

	else
	{
		// If first numeric is 9 then don't allow fourth digit to be > 0
		if(((int)strText[1] - 48 == 9) && (strText[4] > '0'))
			return false;
	}

	// Fifth numeric must be less than or equal to 9
	if(strText[5] < '0' || strText[5] > '9')
		return false;

	else
	{
		// If first numeric is 9 then don't allow fifth digit to be > 0
		if(((int)strText[1] - 48 == 9) && (strText[5] > '0'))
			return false;
	}

	// Third char must be a colon or decimal point
	if(strText[6] != ':')
		return false;



	// If we're using seconds seventh numeric must be less than or equal to 5

	if(strText[7] < '0' || strText[7] > '5')
		return false;

	// If first numeric is 9 then don't allow seventh digit to be > 0
	if(((int)strText[1] - 48 == 9) && (strText[7] > '0'))
		return false;



	// Eighth numeric must be less than or equal to 9
	if(strText[8] < '0' || strText[8] > '9')
		return false;

	// If first numeric is 9 then don't allow eighth digit to be > 0
	if(((int)strText[1] - 48 == 9) && (strText[8] > '0'))
		return false;

    if(nStrLen > 9 )
    {
		if(strText[9] != '.')
			return false;

		if(strText[10] < '0' || strText[10] > '9')
			return false;
    }

	return true;
}




/**--------------------------------------------------------------------------<BR>
CGeoLatLong::ValidateLongValue <BR>

<P>---------------------------------------------------------------------------*/
bool CGeoLatLong::ValidateLongitude(const char* strText)
{
    const int nStrLen = strlen(strText);

    if( nStrLen < 10 || nStrLen > 12 )
        return false;

	if(strText[0] != 'E' && strText[0] != 'W')
		return false;

	// First numeric must be less than 2
	if(strText[1] < '0' || strText[1] > '1')
		return false;

	// Second numeric must be less than or equal to 9
	if(strText[2] < '0' || strText[2] > '9')
		return false;


	// If previous value is 1 then don't alow value > 7
	if(((int)strText[1] - 48 == 1) && (strText[2] > '7'))
		return false;


	// Third numeric must be less than or equal to 9
	if(strText[3] < '0' || strText[3] > '9')
		return false;

	// Fourth char must be a colon
	if(strText[4] != ':')
		return false;

	// Fifth numeric must be less than or equal to 5
	if(strText[5] < '0' || strText[5] > '5')
		return false;

	// Sixth numeric must be less than or equal to 9
	if(strText[6] < '0' || strText[6] > '9')
		return false;

	// Seventh char must be a colon or decimal point
	if(strText[7] != ':')
		return false;


	// If we're using seconds eighth numeric must be less than or equal to 5
	if(strText[8] < '0' || strText[8] > '5')
		return false;


	// Eighth numeric must be less than or equal to 9
	if(strText[9] < '0' || strText[9] > '9')
		return false;

	return true;
}



/// Sets the latitude in degrees
bool CGeoLatLong::SetLatString(const char* strText)
{
	if (ValidateLatitude(strText))
	{
		double dLat = LatitudeToDegrees(strText);
		SetLatDegrees(dLat);
		return true;
	}

	return false;
}

/// Sets the longitude in degrees
bool CGeoLatLong::SetLongString(const char* strText)
{
	if (ValidateLongitude(strText))
	{
		double dLong = LongitudeToDegrees(strText);
		SetLongDegrees(dLong);
		return true;
	}

	return false;

}













































/**--------------------------------------------------------------------------<BR>
CGeoLatLongHeight::CGeoLatLongHeight <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
CGeoLatLongHeight::CGeoLatLongHeight() : CGeoLatLong()
{
	m_dHeight = 0;
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLongHeight::CGeoLatLongHeight <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
CGeoLatLongHeight::CGeoLatLongHeight(double dLatitudeRadians, 
			double dLongitudeRadians, double dAltMetres) : CGeoLatLong(dLatitudeRadians, dLongitudeRadians)
{
	m_dHeight = dAltMetres;
}

/**--------------------------------------------------------------------------<BR>
CGeoLatLongHeight::CGeoLatLongHeight <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
CGeoLatLongHeight::CGeoLatLongHeight(const CGeoLatLongHeight& Other)
{
	m_dHeight = Other.GetHeight();
	m_dLat = Other.GetLat();
	m_dLong = Other.GetLong();
}

/**--------------------------------------------------------------------------<BR>
CGeoLatLongHeight::CGeoLatLongHeight <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
CGeoLatLongHeight::CGeoLatLongHeight(const CGeoLatLong& Other, double dHeight)
{
	m_dLat = Other.GetLat();
	m_dLong = Other.GetLong();
	m_dHeight = dHeight;
}

/**--------------------------------------------------------------------------<BR>
CGeoLatLongHeight::~CGeoLatLongHeight <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CGeoLatLongHeight::~CGeoLatLongHeight()
{


}

/**--------------------------------------------------------------------------<BR>
CGeoLatLong::ConvertTo3DWSG84 <BR>
\brief Conversion to geocentric.
<P>---------------------------------------------------------------------------*/
void CGeoLatLongHeight::GeocentricWSG84(C3DPoint& xyzPoint) const
{
	if(!IsValid())
		return;

	double Sin_Lat = sin(m_dLat);  //  sin(Latitude)
	double Cos_Lat = cos(m_dLat);  //  cos(Latitude)
	double Sin2_Lat = Sin_Lat * Sin_Lat; // Square of sin(Latitude)
	double Rn = conGeocent_Major / (sqrt(1.0e0 - conGeocent_e2 * Sin2_Lat)); //  Earth radius at location

	xyzPoint.x = (Rn + m_dHeight) * Cos_Lat * cos(m_dLong);

	xyzPoint.y = (Rn + m_dHeight) * Cos_Lat * sin(m_dLong);

	xyzPoint.z = ((Rn * (1 - conGeocent_e2)) + m_dHeight) * Sin_Lat;

	return;
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::Geocentric <BR>
\brief Conversion to geocentric.
<P>---------------------------------------------------------------------------*/
void CGeoLatLongHeight::Geocentric(C3DPoint& xyzPoint) const
{
	if(!IsValid())
		return;

	double Sin_Lat = sin(m_dLat);  //  sin(Latitude)
	double Cos_Lat = cos(m_dLat);  //  cos(Latitude)

	xyzPoint.x = (conEARTH_RADIUS_METRES + m_dHeight) * Cos_Lat * cos(m_dLong);

	xyzPoint.y = (conEARTH_RADIUS_METRES + m_dHeight) * Cos_Lat * sin(m_dLong);

	xyzPoint.z = (conEARTH_RADIUS_METRES + m_dHeight) * Sin_Lat;

	return;
}

/**--------------------------------------------------------------------------<BR>
CGeoLatLong::SlantRangeWSG84 <BR>
\brief Slant Range.
<P>---------------------------------------------------------------------------*/
double CGeoLatLongHeight::SlantRangeWSG84(const CGeoLatLongHeight& Other) const
{
	C3DPoint ptThis;

	this->GeocentricWSG84(ptThis);

	C3DPoint ptOther;

	Other.GeocentricWSG84(ptOther);

	return ptThis.Distance(ptOther);

}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::SlantRange <BR>
\brief Slant Range.
<P>---------------------------------------------------------------------------*/
double CGeoLatLongHeight::SlantRange(const CGeoLatLongHeight& Other) const
{
	C3DPoint ptThis;

	this->Geocentric(ptThis);

	C3DPoint ptOther;

	Other.Geocentric(ptOther);

	return ptThis.Distance(ptOther);
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::Range <BR>
\brief Finds the range from one position to another based on sperical earth and
great circles. Range is in metres. Error between this and WSG84 slant range is
less than 1% at ranges up to 2000km. (Spherical Law of cosines).
<P>---------------------------------------------------------------------------*/
double CGeoLatLongHeight::Range(const CGeoLatLongHeight& Other) const
{
	double dLatO = Other.GetLat();
	double dLongO = Other.GetLong();

	if (m_dLat == dLatO && m_dLong == dLongO) return 0;

	double dA1 = sin(m_dLat)*sin(dLatO) + 
		cos(m_dLat)*cos(dLatO)*cos(m_dLong - dLongO);

	double dA = acos(dA1) * (conEARTH_RADIUS_METRES + (m_dHeight+Other.GetHeight()) / 2.0);

	return  dA;
}

/**--------------------------------------------------------------------------<BR>
CGeoLatLong::Range <BR>
\brief Finds the range from one position to another based on sperical earth and
great circles. Range is in metres. Error between this and WSG84 slant range is
less than 1% at ranges up to 2000km. (Spherical Law of cosines).
<P>---------------------------------------------------------------------------*/
double CGeoLatLongHeight::Range(const CGeoLatLong& Other) const
{
	return CGeoLatLong::Range(Other);
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::operator= <BR>
\brief Assignment.
<P>---------------------------------------------------------------------------*/
CGeoLatLongHeight& CGeoLatLongHeight::operator=(const CGeoLatLong& Other)
{
	m_dLat = Other.GetLat();
	m_dLong= Other.GetLong();
	m_dHeight = 0;

	return *this;
}

/**--------------------------------------------------------------------------<BR>
CGeoLatLong::operator= <BR>
\brief Assignment.
<P>---------------------------------------------------------------------------*/
CGeoLatLongHeight& CGeoLatLongHeight::operator=(const CGeoLatLongHeight& Other)
{
	m_dLat = Other.GetLat();
	m_dLong= Other.GetLong();
	m_dHeight = Other.GetHeight();

	return *this;
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::operator== <BR>
\brief Equality.
<P>---------------------------------------------------------------------------*/
bool CGeoLatLongHeight::operator==(CGeoLatLongHeight& Other) const
{
	if ( CGeoLatLong::operator !=(Other))
		return false;

	bool bHeightClose;


	if ( m_dHeight == 0 )
		bHeightClose = Other.GetHeight() == 0;
	else
		bHeightClose = fabs(( Other.GetHeight() - m_dHeight ) / m_dHeight ) 
												< conEqualityTolerance;
	return bHeightClose;

}

/**--------------------------------------------------------------------------<BR>
CGeoLatLong::operator!= <BR>
\brief Inequality.
<P>---------------------------------------------------------------------------*/
bool CGeoLatLongHeight::operator!=(CGeoLatLongHeight& Other) const
{
	return ! this->operator ==(Other);

}

/**--------------------------------------------------------------------------<BR>
CGeoLatLong::operator== <BR>
\brief Equality.
<P>---------------------------------------------------------------------------*/
bool CGeoLatLongHeight::operator==(CGeoLatLong& Other) const
{
	return CGeoLatLong::operator ==(Other);
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::operator!= <BR>
\brief Inequality.
<P>---------------------------------------------------------------------------*/
bool CGeoLatLongHeight::operator!=(CGeoLatLong& Other) const
{
	return  CGeoLatLong::operator !=( Other);
}


/**--------------------------------------------------------------------------<BR>
CGeoLatLong::Set <BR>
\brief Assignment.
<P>---------------------------------------------------------------------------*/
void CGeoLatLongHeight::Set(double dHeadingRadians, double dRange, 
							const CGeoLatLong& LLOrigin)
{
	CGeoLatLong::Set( dHeadingRadians, dRange,  LLOrigin);
	m_dHeight = 0;
}


