/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file Geodetic.h
\brief File for the CGeoLatLong and CGeoLatLongHeight classes which contains 
data for latitude, longitude and height. 
<P>---------------------------------------------------------------------------*/

#ifndef _GEOLIB_GEODETIC_H 
#define _GEOLIB_GEODETIC_H

#include "Constants.h"

class C2DPoint;
class C3DPoint;

#ifdef _PROJ_EXPORTING
	#define PROJ_DECLSPEC		__declspec(dllexport)
#else 
	#ifdef _STATIC
		#define PROJ_DECLSPEC
	#else
		#define PROJ_DECLSPEC		__declspec(dllimport)
	#endif
#endif

/**--------------------------------------------------------------------------<BR>
\class CGeoLatLong
\brief Class with data for latitude, and longitude. Functions to convert to 
Geocentric co-ordinates and 2D. Units in metres.
<P>---------------------------------------------------------------------------*/
class PROJ_DECLSPEC CGeoLatLong  
{
public:
	/// Constructor
	CGeoLatLong();
	/// Constructor
	CGeoLatLong(double dLatitudeRadians, double dLongitudeRadians);
	/// Constructor
	CGeoLatLong(const CGeoLatLong& Other);
	/// Destructor
	~CGeoLatLong();

	/// Checks the latitude and longitude are within the bounds
	bool IsValid(void) const;
	/// Sets the latitude in degrees
	void SetLatDegrees(double dLatDegrees) 
		{m_dLat = dLatDegrees * conRadiansPerDegree;}
	/// Sets the longitude in degrees
	void SetLongDegrees(double dLongDegrees)
		{m_dLong = dLongDegrees * conRadiansPerDegree;}

	/// Sets the latitude in degrees
	bool SetLatString(const char* strText);
	/// Sets the longitude in degrees
	bool SetLongString(const char* strText);



	/// Gets the latitude in degrees
	double GetLatDegrees() const 
		{return m_dLat * conDegreesPerRadian;}
	/// Gets the longitude in degrees
	double GetLongDegrees() const 
		{return m_dLong * conDegreesPerRadian;}
	/// Sets the latitude in radians
	void SetLat(double dLatRadians) 
		{m_dLat = dLatRadians; }
	/// Sets the longitude in radians
	void SetLong(double dLongRadians)
		{m_dLong = dLongRadians;}
	/// Gets the latitude in radians
	double GetLat() const {return m_dLat;}
	/// Gets the longitude in radians
	double GetLong() const {return m_dLong;}

	/// Latitude as a string. Target must be at least 10 chars long.
	void GetLatString(char* dest) const;
	///// Longitude as a string. Target must be at least 11 chars long.
	void GetLongString(char* dest) const;
	/// Assignment
	CGeoLatLong& operator=(const CGeoLatLong& Other);
	/// Equality
	bool operator==(CGeoLatLong& Other) const;
	/// Inequality
	bool operator!=(CGeoLatLong& Other) const;

	/// Great circle range to another, ignores height
	double Range(const CGeoLatLong& Other) const;
	/// Heading from this to the other in Radians
	double Heading(const CGeoLatLong& Other) const;
	/// Returns the range and the heading to the other point.
	void RangeAndHeading(const CGeoLatLong& Other, double& dRangeMetres, double& dHeadingRadians) const;

	/// Sets the data by using a heading and range from another
	void  Set(double dHeadingRadians, double dRange, const CGeoLatLong& Origin);

	/// Adds the latitude and longitude from the other to this
	void Rotate(const CGeoLatLong& Other);

	/// Converts to Geocentric using WSG84 algorithm
	void GeocentricWSG84(C3DPoint& xyzPoint) const;
	/// Converts to Geocentric with assumed sherical earth
	void Geocentric(C3DPoint& xyzPoint) const;
	/// The direct range with WSG84 modelled elipsical earth.
	double SlantRangeWSG84(const CGeoLatLong& Other) const;
	/// The direct range with spherical earth.
	double SlantRange(const CGeoLatLong& Other) const;




	/// Latitude as a string.
	const char* GetLatString() const;
	///// Longitude as a string. 
	const char* GetLongString() const;

	/// Latitude as a string.
	static const char* GetLatString(double dLatitudeDegrees);
	///// Longitude as a string. 
	static const char* GetLongString(double dLongitudeDegrees);



	/// Converts range to slant range based on spherical earth.
	static double RangeToSlantRange(double dRange);
	/// Converts slant range to range based on spherical earth.
	static double SlantRangeToRange(double dSlantRange);

	static bool ValidateLatitude(const char* strText);

	static bool ValidateLongitude(const char* strText);

	static double LongitudeToDegrees(const char* pText);

	static double LatitudeToDegrees(const char* pText);

protected:
	/// Latitude
	double m_dLat;
	/// Longitude
	double m_dLong;
};


/**--------------------------------------------------------------------------<BR>
\class CGeoLatLongHeight
\brief Class with data for latitude, longitude and height. Functions to convert to 
Geocentric co-ordinates and 2D.
<P>---------------------------------------------------------------------------*/
class PROJ_DECLSPEC CGeoLatLongHeight : public CGeoLatLong
{
public:
	/// Constructor
	CGeoLatLongHeight();
	/// Constructor
	CGeoLatLongHeight(double dLatitudeRadians, double dLongitudeRadians, double dAltMetres);
	/// Constructor
	CGeoLatLongHeight(const CGeoLatLongHeight& Other);
	/// Constructor
	CGeoLatLongHeight(const CGeoLatLong& Other, double dHeight = 0);
	/// Destructor
	~CGeoLatLongHeight();
	/// Returns this as a simple lat long.
	CGeoLatLong& AsLatLong(void) {return reinterpret_cast<CGeoLatLong&>(*this);}

	/// Converts to Geocentric using WSG84 algorithm
	void GeocentricWSG84(C3DPoint& xyzPoint) const;
	/// Converts to Geocentric with assumed sherical earth
	void Geocentric(C3DPoint& xyzPoint) const;
	/// The direct range with elipsical earth.
	double SlantRangeWSG84(const CGeoLatLongHeight& Other) const;
	/// The direct range with spherical earth.
	double SlantRange(const CGeoLatLongHeight& Other) const;

	/// Great circle range to another using average height of the 2.
	double Range(const CGeoLatLongHeight& Other) const;
	/// Great circle range to another, ignores height
	double Range(const CGeoLatLong& Other) const;

	/// Assignment
	CGeoLatLongHeight& operator=(const CGeoLatLong& Other);
	/// Assignment
	CGeoLatLongHeight& operator=(const CGeoLatLongHeight& Other);

	/// Equality
	bool operator==(CGeoLatLongHeight& Other) const;
	/// Inequality
	bool operator!=(CGeoLatLongHeight& Other) const;
	/// Equality
	bool operator==(CGeoLatLong& Other) const;
	/// Inequality
	bool operator!=(CGeoLatLong& Other) const;
	/// Sets using range and heading from an origin.
	void Set(double dHeadingRadians, double dRange, const CGeoLatLong& LLOrigin);
	/// Gets the height in metres.
	double GetHeight(void) const {return m_dHeight;}
	/// Gets the altitude
	void SetHeight(double dHeightMetres) { m_dHeight = dHeightMetres;}

protected:
	/// The height in metres.
	double m_dHeight;

};

#endif
