/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file LambertAzimuthalEqualArea.cpp
\brief Implementation file for a CLambertAzimuthalEqualArea class.

Implementation file for a CLambertAzimuthalEqualArea class.
<P>---------------------------------------------------------------------------*/




#include "LambertAzimuthalEqualArea.h"
#include "Geodetic.h"
#include "StdAfx.h"

/**--------------------------------------------------------------------------<BR>
LambertAzimuthalEqualArea::LambertAzimuthalEqualArea <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CLambertAzimuthalEqualArea::CLambertAzimuthalEqualArea(void)   : CProjection(eLambertAzimuthalEqualArea)
{
	m_dStandardParallel = 0;

	m_dCentralLongitude = 0;
}

/**--------------------------------------------------------------------------<BR>
CLambertAzimuthalEqualArea::~CLambertAzimuthalEqualArea <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CLambertAzimuthalEqualArea::~CLambertAzimuthalEqualArea(void)
{
}

/**--------------------------------------------------------------------------<BR>
CLambertAzimuthalEqualArea::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CLambertAzimuthalEqualArea::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	double sin_SP = sin(m_dStandardParallel);
	double sin_lat = sin(dLatY);

	double cos_SP = cos(m_dStandardParallel);
	double cos_lat = cos(dLatY);
	double cos_Dlong = cos(dLongX - m_dCentralLongitude);

	double dK = sqrt(2.0 / ( 1 + sin_SP*sin_lat + cos_SP*cos_lat*cos_Dlong));

	dLongX = dK * cos_lat * sin(dLongX - m_dCentralLongitude);

	dLatY = dK*( cos_SP* sin_lat - sin_SP * cos_lat * cos_Dlong);
}


/**--------------------------------------------------------------------------<BR>
CLambertAzimuthalEqualArea::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CLambertAzimuthalEqualArea::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();
	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CLambertAzimuthalEqualArea::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CLambertAzimuthalEqualArea::InverseProject(double& dLatY, double& dLongX) const
{
	double P1 = dLongX * dLongX + dLatY * dLatY;

	double P = sqrt( P1 );
	
	double C = 2 * asin( 0.5 * P);

	double cos_C = cos(C);

	double sin_C = sin(C);

	double sin_SP = -sin(m_dStandardParallel);

	double cos_SP = cos(m_dStandardParallel);

	double dLat = asin( cos_C* sin_SP + (dLatY * sin_C * cos_SP) / P);

	double dLong;

	if (P1 < 2)
	{
			dLong = m_dCentralLongitude + atan(dLongX * sin_C /
								(P *cos_SP * cos_C - dLatY * sin_SP * sin_C)  );
	}
	else
	{
		if (dLongX	> 0 )
			dLong = m_dCentralLongitude + conPI + atan( dLongX * sin_C /
									(P *cos_SP * cos_C - dLatY* sin_SP * sin_C)  );
		else
			dLong = m_dCentralLongitude - conPI + atan( dLongX * sin_C /
											(P *cos_SP * cos_C - dLatY * sin_SP * sin_C)  );
	}

	dLatY = dLat * conDegreesPerRadian;

	dLongX = dLong * conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CLambertAzimuthalEqualArea::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CLambertAzimuthalEqualArea::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = dY;

	double dLong = dX;

	InverseProject(dLat, dLong);

	rLatLong.SetLatDegrees(dLat);

	rLatLong.SetLongDegrees(dLong);
}




/**--------------------------------------------------------------------------<BR>
CLambertAzimuthalEqualArea::SetStandardParallel2 <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CLambertAzimuthalEqualArea::SetOrigin(double dLat, double dLong)
{
	m_dStandardParallel = dLat * conRadiansPerDegree;

	m_dCentralLongitude = dLong * conRadiansPerDegree;

}
