/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file LambertConformalConic.cpp
\brief Implementation file for a CLambertConformalConic class.

Implementation file for a CLambertConformalConic class.
<P>---------------------------------------------------------------------------*/




#include "LambertConformalConic.h"
#include "Geodetic.h"
#include "StdAfx.h"



CLambertConformalConic::CLambertConformalConic(void)  : CProjection(eLambertConformalConic)
{
	m_dStandardParallel1 = 50 * conRadiansPerDegree;

	m_dStandardParallel2 = -10 * conRadiansPerDegree;

	m_dOriginLat = 0;

	m_dOriginLong = 0;

	CalculateConstants();
}

CLambertConformalConic::~CLambertConformalConic(void)
{

}


/**--------------------------------------------------------------------------<BR>
CLambertConformalConic::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CLambertConformalConic::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;


	double p = m_dF * pow( 1 / tan(conQUARTPI + dLatY / 2), m_dn);

	double p0 = m_dF * pow( 1 / tan(conQUARTPI + m_dOriginLat / 2), m_dn);

	double x = p* sin( m_dn * (dLongX - m_dOriginLong));

	double y = p0 - p * cos( m_dn * (dLongX  - m_dOriginLong));

	dLongX = x;

	dLatY = y;
}


/**--------------------------------------------------------------------------<BR>
CLambertConformalConic::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CLambertConformalConic::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();
	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CLambertConformalConic::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CLambertConformalConic::InverseProject(double& dLatY, double& dLongX) const
{
	double p0 = m_dF * pow( 1 / tan(conQUARTPI + m_dOriginLat / 2), m_dn);


	double p = sqrt(dLongX * dLongX + (p0 - dLatY)* (p0 - dLatY));
	if (m_dn < 0)
		p = -p;

	double theta = atan(dLongX / (p0 - dLatY));

	dLatY = 2 * atan( pow( m_dF / p, 1/m_dn) ) - conHALFPI;

	dLongX = m_dOriginLong + theta / m_dn;



	dLatY *= conDegreesPerRadian;

	dLongX *= conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CLambertConformalConic::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CLambertConformalConic::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLatY = dY;

	double dLongX = dX;

	InverseProject(dLatY, dLongX);

	rLatLong.SetLat(dLatY);

	rLatLong.SetLong(dLongX);
}

/**--------------------------------------------------------------------------<BR>
CLambertConformalConic::SetStandardParallel2 <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CLambertConformalConic::SetStandardParallels(double dStandardParallel, double dStandardParalle2)
{
	m_dStandardParallel1 = dStandardParallel * conRadiansPerDegree;

	m_dStandardParallel2 = dStandardParalle2 * conRadiansPerDegree;

	CalculateConstants();
}


/**--------------------------------------------------------------------------<BR>
CLambertConformalConic::SetStandardParallel2 <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CLambertConformalConic::SetOrigin(double dLat, double dLong)
{
	m_dOriginLat = dLat * conRadiansPerDegree;

	m_dOriginLong = dLong * conRadiansPerDegree;

	CalculateConstants();
}

/**--------------------------------------------------------------------------<BR>
CLambertConformalConic::CalculateConstants <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CLambertConformalConic::CalculateConstants()
{
	double cos_phi1 = cos(m_dStandardParallel1);

	double n1 = log( cos_phi1 * (1 / cos (m_dStandardParallel2)));
	double n2 = tan( conQUARTPI + m_dStandardParallel2 / 2);
	double n3 = 1 / (tan( conQUARTPI + m_dStandardParallel1 / 2));
	m_dn = n1 / log(n2 * n3 );
	m_dF = cos_phi1 * pow( tan( conQUARTPI + m_dStandardParallel1 / 2), m_dn) / m_dn;


}