/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file Mollweide.cpp
\brief Implementation file for a CMollweide class.

Implementation file for a CMollweide class.
<P>---------------------------------------------------------------------------*/



#include "Mollweide.h"
#include "Geodetic.h"
#include "StdAfx.h"



CMollweide::CMollweide(void) : CProjection(eMollweide)
{
	m_dStandardLongitude = 0;
}

CMollweide::~CMollweide(void)
{
}



/**--------------------------------------------------------------------------<BR>
CMollweide::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CMollweide::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	static const double dTolerance = 0.00000001;

	double theta1 = 2* asin( 2* dLatY / conPI); // Iteration start

	double theta0 = theta1 + dTolerance * 10.0;

	static const unsigned int nMaxIt = 50;

	unsigned int nIt = 0; 

	double sin_lat = sin (dLatY);

	while ( fabs(theta1 - theta0) > dTolerance)
	{
		theta0 = theta1;

		theta1 -= (theta0 + sin(theta0) - conPI * sin_lat) /
					(1 + cos(theta0));

		nIt++;
		if(nIt == nMaxIt)
		{
		//	assert(0);
			return;
		}
	}

	theta1 = theta1 / 2.0;

	dLongX = 2.0 * conRoot2 * (dLongX  - m_dStandardLongitude)* cos (theta1) / conPI;

	dLatY = conRoot2 * sin( theta1 );

}



/**--------------------------------------------------------------------------<BR>
CMollweide::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CMollweide::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();

	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CMollweide::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CMollweide::InverseProject(double& dLatY, double& dLongX) const
{
	double theta = asin(dLatY / conRoot2);

	dLatY = asin( (2 * theta + sin( 2 * theta )) / conPI);

	dLongX = m_dStandardLongitude + conPI * dLongX / (2 * conRoot2 * cos( theta ));

	dLatY *= conDegreesPerRadian;

	dLongX *= conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CMollweide::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CMollweide::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLatY = dY;

	double dLongX = dX;

	InverseProject(dLatY, dLongX);

	rLatLong.SetLat(dLatY);

	rLatLong.SetLong(dLongX);
}


/**--------------------------------------------------------------------------<BR>
CMollweide::SetStandardLongitude <BR>
\brief
<P>---------------------------------------------------------------------------*/
void CMollweide::SetStandardLongitude(double dStandardLongitude)
{
	m_dStandardLongitude = dStandardLongitude * conRadiansPerDegree;
}