/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file Polyconic.cpp
\brief Implementation file for a CPolyconic class.

Implementation file for a CPolyconic class.
<P>---------------------------------------------------------------------------*/




#include "Polyconic.h"
#include "Geodetic.h"
#include "StdAfx.h"

/**--------------------------------------------------------------------------<BR>
Polyconic::Polyconic <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CPolyconic::CPolyconic(void)   : CProjection(ePolyconic)
{
	m_dStandardLatitude = 0;

	m_dStandardLongitude = 0;
}

/**--------------------------------------------------------------------------<BR>
CPolyconic::~CPolyconic <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CPolyconic::~CPolyconic(void)
{
}

/**--------------------------------------------------------------------------<BR>
CPolyconic::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CPolyconic::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	double E = (dLongX - m_dStandardLongitude) * sin(dLatY);

	dLongX = sin(E) / tan (dLatY);

	dLatY = (dLatY - m_dStandardLatitude ) + (1 - cos(E)) / tan(dLatY);
}


/**--------------------------------------------------------------------------<BR>
CPolyconic::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CPolyconic::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();
	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CPolyconic::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CPolyconic::InverseProject(double& dLatY, double& dLongX) const
{
	double A = m_dStandardLatitude + dLatY;

	double B = dLongX * dLongX + A*A;

	static const double dTolerance = 0.00000001;

	double theta1 = A;
	if (dLongX != 0)
		theta1 = A * fabs(dLongX)* 0.1; // Iteration start

	double theta0 = theta1 + dTolerance * 10.0;

	static const unsigned int nMaxIt = 500;

	unsigned int nIt = 0; 

	while ( fabs(theta1 - theta0) > dTolerance)
	{
		theta0 = theta1;

		double tan_theta = tan(theta0);

		theta1 -= (   A*(theta0*tan_theta + 1) - theta0 - 0.5*(theta0*theta0 + B)*tan(theta0)  )   /
								( ( theta0 - A) / tan(theta0) - 1 );

		nIt++;
		if(nIt == nMaxIt)
		{
		//	assert(0);
			break;
		}
	}

	dLatY = theta1;

	dLongX = asin(dLongX * tan(theta1) ) / sin(theta1) + m_dStandardLongitude;

	dLatY *= conDegreesPerRadian;

	dLongX *= conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CPolyconic::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CPolyconic::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = dY;

	double dLong = dX;

	InverseProject(dLat, dLong);

	rLatLong.SetLatDegrees(dLat);

	rLatLong.SetLongDegrees(dLong);
}




/**--------------------------------------------------------------------------<BR>
CPolyconic::SetStandardParallel2 <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CPolyconic::SetOrigin(double dLat, double dLong)
{
	m_dStandardLatitude = dLat * conRadiansPerDegree;

	m_dStandardLongitude = dLong * conRadiansPerDegree;

}
