/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file Stereographic.cpp
\brief Implementation file for a CStereographic class.

Implementation file for a CStereographic class.
<P>---------------------------------------------------------------------------*/



#include "Stereographic.h"
#include "Geodetic.h"
#include "StdAfx.h"

/**--------------------------------------------------------------------------<BR>
Stereographic::Stereographic <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CStereographic::CStereographic(void)   : CProjection(eStereographic)
{
	m_dStandardLatitude = 0;

	m_dStandardLongitude = 0;
}

/**--------------------------------------------------------------------------<BR>
CStereographic::~CStereographic <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CStereographic::~CStereographic(void)
{
}

/**--------------------------------------------------------------------------<BR>
CStereographic::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CStereographic::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	double sin_olat = sin(m_dStandardLatitude);
	double sin_lat = sin(dLatY);
	double cos_olat = cos(m_dStandardLatitude);
	double cos_lat = cos(dLatY);
	double cos_dlong = cos(dLongX - m_dStandardLongitude);

	double k = 2* conEARTH_RADIUS_METRES /
		( 1 + sin_olat * sin_lat + cos_olat * cos_lat * cos_dlong);

	dLongX = k * cos_lat * sin(dLongX  - m_dStandardLongitude);

	dLatY = k * (cos_olat * sin_lat - sin_olat * cos_lat * cos_dlong);

}


/**--------------------------------------------------------------------------<BR>
CStereographic::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CStereographic::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();
	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CStereographic::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CStereographic::InverseProject(double& dLatY, double& dLongX) const
{
	double p = sqrt(dLongX * dLongX + dLatY * dLatY);

	double c = 2 * atan2(p, 2 * conEARTH_RADIUS_METRES);
	
	double cos_c = cos(c);
	double sin_c = sin(c);
	double sin_olat = sin(m_dStandardLatitude);

	double cos_olat = cos(m_dStandardLatitude);

	double dLat = asin( cos_c * sin_olat + dLatY * sin_c * cos_olat / p);

	double dLong = atan2(dLongX * sin_c, p * cos_olat * cos_c - dLatY * sin_olat * sin_c);

	dLong += m_dStandardLongitude;


	dLatY = dLat * conDegreesPerRadian;

	dLongX = dLong * conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CStereographic::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CStereographic::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = dY;

	double dLong = dX;

	InverseProject(dLat, dLong);

	rLatLong.SetLatDegrees(dLat);

	rLatLong.SetLongDegrees(dLong);
}




/**--------------------------------------------------------------------------<BR>
CStereographic::SetStandardParallel2 <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CStereographic::SetOrigin(double dLat, double dLong)
{
	m_dStandardLatitude = dLat * conRadiansPerDegree;

	m_dStandardLongitude = dLong * conRadiansPerDegree;

}
