/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file CVanDerGrinten.cpp
\brief Implementation file for a CVanDerGrinten class.

Implementation file for a CVanDerGrinten class.
<P>---------------------------------------------------------------------------*/


#include "VanDerGrinten.h"
#include "Geodetic.h"
#include "StdAfx.h"




/**--------------------------------------------------------------------------<BR>
CVanDerGrinten::CVanDerGrinten <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
CVanDerGrinten::CVanDerGrinten(void) : CProjection(eVanDerGrinten)
{
	m_dStandardLongitude = 0;
}


/**--------------------------------------------------------------------------<BR>
CVanDerGrinten::CVanDerGrinten <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
CVanDerGrinten::~CVanDerGrinten(void)
{
}

/**--------------------------------------------------------------------------<BR>
CVanDerGrinten::Project <BR>
\brief Constructor.
<P>---------------------------------------------------------------------------*/
void CVanDerGrinten::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;

	double A1 = conPI / (dLongX  - m_dStandardLongitude);

	double A = 0.5 * fabs( A1 - 1.0 / A1);

	double theta = asin( fabs(2.0* dLatY / conPI) );

	double cos_theta = cos(theta);

	double sin_theta = sin(theta);

	double G = cos_theta / ( sin_theta + cos_theta - 1.0);

	double P = G*(2.0/ sin(theta) - 1.0);

	double P2 = P*P;
	double A2 = A*A;

	double Q = A2 + G;


	double P2A2 = P2 + A2;
	double x0 = G - P2;
	double x1 = A * x0;
	double x2 = A2 * x0 * x0 - (P2A2)*(G*G - P2);
	double x = conPI * ( x1 + sqrt( x2 ) ) /
					P2A2;

	if (dLongX  < m_dStandardLongitude )
		x = -x;


	double y = conPI * fabs( P*Q - A* sqrt( (A2 + 1)*(P2A2) - Q*Q)   )  /
					P2A2;

	if (dLatY < 0)
		y = -y;


	dLongX = x;
	dLatY = y;


}

/**--------------------------------------------------------------------------<BR>
CVanDerGrinten::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CVanDerGrinten::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();

	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CVanDerGrinten::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store 
the result.	
<P>---------------------------------------------------------------------------*/
void CVanDerGrinten::InverseProject(double& dLatY, double& dLongX) const
{
	double X = dLongX / conPI;
	double X2 = X * X;
	double Y = dLatY / conPI;
	double Y2 = Y * Y;
	double X2Y2 = X2 + Y2;
	double c1 = -fabs(Y) * (1 + X2Y2);
	double c2 = c1 - 2.0 * Y2 + X2;
	double c3 = -2.0 * c1 + 1 + 2 * Y2 + X2Y2* X2Y2;
	double d = Y2 / c3 + (2* c2*c2*c2 / (c3*c3*c3) - 9 * c1*c2 / (c3*c3)) / 27.0;
	double a1 = (c1 - c2*c2 / (3*c3)) / c3;
	double m1 = 2 * sqrt( - a1 / 3.0);
	double theta1 = acos(3.0 * d / ( a1 *m1)) / 3.0;

	double dLat = conPI * ( -m1 * cos(theta1 + conTHIRDPI) - c2 / (3*c3));
	if (dLatY < 0)
		dLat = -dLat;

	double dLong = conPI *( X2 + Y2 - 1 + sqrt( 1.0 + 2.0 * ( X2 - Y2) + X2Y2 *  X2Y2 )) / 
					(2.0 * X);

	dLong += m_dStandardLongitude;

	if (dLong > conPI)
		dLong = conPI;
	if (dLong < -conPI)
		dLong = -conPI;

	if (dLat > conHALFPI)
		dLat = conHALFPI;
	if (dLat < -conHALFPI)
		dLat = -conHALFPI;


	dLatY = dLat * conDegreesPerRadian;

	dLongX = dLong * conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CVanDerGrinten::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CVanDerGrinten::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = dY;

	double dLong = dX;

	InverseProject(dLat, dLong);

	rLatLong.SetLatDegrees(dLat);

	rLatLong.SetLongDegrees(dLong);
}

/**--------------------------------------------------------------------------<BR>
CVanDerGrinten::SetStandardLongitude <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CVanDerGrinten::SetStandardLongitude(double dStandardLongitude)
{
	m_dStandardLongitude = dStandardLongitude * conRadiansPerDegree;
}