/*---------------------------------------------------------------------------
Copyright (C) GeoLib.
This code is used under license from GeoLib (www.geolib.co.uk). This or
any modified versions of this cannot be resold to any other party.
---------------------------------------------------------------------------*/


/**--------------------------------------------------------------------------<BR>
\file VerticalPerspective.cpp
\brief Implementation file for a CVerticalPerspective class.

Implementation file for a CVerticalPerspective class.
<P>---------------------------------------------------------------------------*/



#include "VerticalPerspective.h"
#include "Geodetic.h"
#include "StdAfx.h"

/**--------------------------------------------------------------------------<BR>
VerticalPerspective::VerticalPerspective <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CVerticalPerspective::CVerticalPerspective(void)   : CProjection(eVerticalPerspective)
{
	m_dStandardLatitude = 0;

	m_dStandardLongitude = 0;

	m_P = 10.0;
}

/**--------------------------------------------------------------------------<BR>
CVerticalPerspective::~CVerticalPerspective <BR>
\brief Destructor.
<P>---------------------------------------------------------------------------*/
CVerticalPerspective::~CVerticalPerspective(void)
{
}

/**--------------------------------------------------------------------------<BR>
CVerticalPerspective::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the 
result.
<P>---------------------------------------------------------------------------*/
void CVerticalPerspective::Project(double& dLatY, double& dLongX) const
{
	dLatY *= conRadiansPerDegree;

	dLongX *= conRadiansPerDegree;


	double sin_lat = sin(dLatY);

	double cos_lat = cos(dLatY);

	double sin_olat = sin(m_dStandardLatitude);

	double cos_olat = cos(m_dStandardLatitude);

	double cos_dlong = cos(dLongX  - m_dStandardLongitude);

	double cos_c = sin_olat * sin_lat + cos_olat * cos_lat * cos_dlong;

	double k = (m_P - 1) / (m_P - cos_c);

	dLongX = k * cos_lat * sin(dLongX  - m_dStandardLongitude);

	dLatY = k * ( cos_olat * sin_lat - sin_olat * cos_lat * cos_dlong);

//	if (pBack != NULL)
	//	*pBack = cos_c < (1 / P);


}


/**--------------------------------------------------------------------------<BR>
CVerticalPerspective::Project <BR>
\brief Project the given lat long to x, y using the input parameters to store the result and retaining 
the lat long in the class passed.
<P>---------------------------------------------------------------------------*/
void CVerticalPerspective::Project(const CGeoLatLong& rLatLong, double& dx, double& dy) const
{
	dy = rLatLong.GetLat();
	dx = rLatLong.GetLong();

	Project(dy, dx);
}

/**--------------------------------------------------------------------------<BR>
CVerticalPerspective::InverseProject <BR>
\brief Project the given x y to lat long using the input parameters to store the result.	
<P>---------------------------------------------------------------------------*/
void CVerticalPerspective::InverseProject(double& dLatY, double& dLongX) const
{

	// TO DO - Need proper inverse projection. Current inverse is valid for infinite P only
	double dHeading = atan2(dLongX,dLatY);

	double dHRange = sqrt(dLongX * dLongX + dLatY * dLatY);

	// alpha is the angular Distance travelled round the earths surface
	double alpha = asin( dHRange );

//	double k = (P - 1) / (P - alpha);

	double sin_alpha = dHRange;
	double cos_alpha = cos(alpha);

	double latO = m_dStandardLatitude;
	double longO = m_dStandardLongitude;

	double sin_Olat = sin(latO);
	double cos_Olat = cos(latO);

	double dLat = asin(  sin_Olat * cos_alpha + cos_Olat * sin_alpha * cos(dHeading)  );

	double dLong = longO + atan2(sin(dHeading) * sin_alpha * cos_Olat , cos_alpha - sin_Olat * sin(dLat));

	while (dLong > conPI) 
		dLong -= conTWOPI;
	while (dLong < -conPI) 
		dLong += conTWOPI;


	dLatY = dLat * conDegreesPerRadian;

	dLongX = dLong * conDegreesPerRadian;
}


/**--------------------------------------------------------------------------<BR>
CVerticalPerspective::InverseProject <BR>
\brief Project the given x y to lat long using the input lat long class to get the result.
<P>---------------------------------------------------------------------------*/
void CVerticalPerspective::InverseProject(CGeoLatLong& rLatLong, const double& dX, const double& dY) const
{
	double dLat = dY;

	double dLong = dX;

	InverseProject(dLat, dLong);

	rLatLong.SetLatDegrees(dLat);

	rLatLong.SetLongDegrees(dLong);
}




/**--------------------------------------------------------------------------<BR>
CVerticalPerspective::SetStandardParallel2 <BR>
\brief 
<P>---------------------------------------------------------------------------*/
void CVerticalPerspective::SetOrigin(double dLat, double dLong)
{
	m_dStandardLatitude = dLat * conRadiansPerDegree;

	m_dStandardLongitude = dLong * conRadiansPerDegree;
}



void CVerticalPerspective::SetPointOfPerspective(float p)
{
	m_P = p;

}